 #define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <lsapi\lsapi.h>
#include "modulelog.h"

#ifndef NLOG

ModuleLog::ModuleLog(str sSettingPrefix, str sModuleName, str sModuleVersion)
  : sFileName_(NULL), nIndent_(0)
{
  bufstr(MAX_LINE_LENGTH) sName;
  bufstr(256) sKey;

  /* try $<modulename>LogFile$                        */ 
  sKey << sSettingPrefix << S("LogFile");
  if (GetRCString(sKey.csafe(), sName.begin(), NULL, sName.capacity()))
  {
    sName.fit();
  }
  else
  {
    /* try $LSLogPath$\<modulename>-<version>.log       */ 
    if (GetRCString(TEXT("LSLogPath"), sName.begin(), NULL, sName.capacity()))
    {
      sName.fit();
      if (sName[-1]!='\\' && sName[-1]!='/')
        sName << TEXT('\\');
    }
    /* try $LiteStepDir$logs\<modulename>-<version>.log */ 
    else
    {
      GetRCString(TEXT("LiteStepDir"), sName.begin(), TEXT("c:\\"), sName.capacity());
      sName.fit();
      sName << S("logs\\");
    }
    CreateDirectory(sName.csafe(), NULL);
    sName << sModuleName << TEXT('-') << sModuleVersion << S(".log");
  }

  /* copy to heap allocated space                     */ 
  bufstr sAllocName(new TCHAR[sName.size()+1], sName.size()+1);
  sAllocName << sName;
  sFileName_ = sAllocName.csafe();
}

void ModuleLog::WriteLine(str sLine, bool bTimeStamp)
{
  if (!sFileName_)
    return;

  HANDLE hFile = CreateFile(sFileName_, FILE_APPEND_DATA, FILE_SHARE_READ,
    NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

  if (hFile == NULL)
  {
    bufstr(4096) sMessage;
    LPSTR values[] =
    {
      (LPSTR)sFileName_,
      (LPSTR)sLine.begin(),
    };

    FormatMessage(
      FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY|60,
      TEXT("Unable to open log file:%n%1%n%nMessage was:%n%2%n%nDisable logging?"),
      NULL,NULL,sMessage.begin(),sMessage.capacity(),values);

    /* technically should do sMessage.fit(), but don't need it */ 

    if (MessageBox(NULL, sMessage.begin(), TEXT("Module Log"), MB_YESNO|MB_ICONERROR|MB_TOPMOST)
      == IDYES)
    {
      delete[] const_cast<TCHAR *>(sFileName_);
      sFileName_ = NULL;
    }
    return;
  }

  SetFilePointer(hFile, 0, NULL, FILE_END);

  DWORD nBytesWritten = 0;

  if (nIndent_)
    WriteFile(hFile, TEXT("                "), (nIndent_&15), &nBytesWritten, NULL);

  if (bTimeStamp)
  {
    /* date/time stamp */ 
    SYSTEMTIME SystemTime;
    GetLocalTime(&SystemTime);

    LPSTR values[] =
    {
      (LPSTR)SystemTime.wYear, (LPSTR)SystemTime.wMonth,
      (LPSTR)SystemTime.wDay, (LPSTR)SystemTime.wHour,
      (LPSTR)SystemTime.wMinute, (LPSTR)SystemTime.wSecond,
      (LPSTR)SystemTime.wMilliseconds
    };

    const unsigned nTimeStamp = 9+(5+3+2)+1+(3+3+3+3)+3;
    TCHAR sTimeStamp[nTimeStamp+1];
    FormatMessage(
      FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
      TEXT("-------- %1!04d!-%2!02d!-%3!02d! %4!02d!:%5!02d!:%6!02d!.%7!03d! : "),
      NULL,NULL,sTimeStamp,nTimeStamp+1,values);
    WriteFile(hFile, sTimeStamp, nTimeStamp, &nBytesWritten, NULL);
  }

  const BOOL bWriteOk = WriteFile(hFile, sLine.begin(), sLine.size(), &nBytesWritten, NULL);

  WriteFile(hFile, TEXT("\x0d\x0a"), 2, &nBytesWritten, NULL);

  CloseHandle(hFile);
}

//
// C interface
//

ModuleLog *g_ModuleLog = NULL;

void ModuleLogInit(const char *sSettingPrefix, unsigned nSettingPrefix,
                   const char *sModuleName, unsigned nModuleName,
                   const char *sModuleVersion, unsigned nModuleVersion)
{
  g_ModuleLog = new ModuleLog(str(sSettingPrefix, nSettingPrefix),
                              str(sModuleName, nModuleName),
                              str(sModuleVersion, nModuleVersion));
}

void ModuleLogQuit(void)
{
  delete g_ModuleLog;
}

void ModuleLogWriteLine(const char *sLine, unsigned nLine)
{
  g_ModuleLog->WriteLine(str(sLine, nLine), false);
}

void ModuleLogWriteLineTimeStamp(const char *sLine, unsigned nLine)
{
  g_ModuleLog->WriteLine(str(sLine, nLine), true);
}

void ModuleLogIndent(int vDif)
{
  g_ModuleLog->Indent(vDif);
}

#endif /* NLOG - empty file */ 
